package Config::Find::Win98;

our $VERSION = '0.01';

use strict;
use warnings;

use Config::Find::WinAny

our @ISA = qw(Config::Find::WinAny);

1;
__END__

=head1 NAME

Config::Find::Win98 - Win98 idiosyncrasies for Config::Find

=head1 SYNOPSIS

  # don't use Config::Find::Win98;
  use Config::Find;

=head1 ABSTRACT

Implements Win98 specific features for Config::Find

=head1 DESCRIPTION

Stub documentation for Config::Find::Win98, created by h2xs.


=head2 EXPORT

None by default.



=head1 SEE ALSO

L<Config::Find>, L<Config::Find::WinAny>, L<Config::Find::Any>

=head1 AUTHOR

Salvador Fandiņo, E<lt>sfandino@yahoo.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2003 by Salvador Fandiņo

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut
